/*----------------------------------------------------------
   Puts mail into Weasel's "forward" directory, so that it
   will be sent out the next time Weasel is on line.
   (If Weasel is already on line, it will check this
   directory every few minutes for new items.)

           Author:       Peter Moylan (peter@ee.newcastle.edu.au)
           Started:      13 May 2000
           Last revised: 15 February 2001

   Usage:
           qmail -f sender@localdomain -t recipient@remotedomain filename

           where the argument after the '-f' specifies the sender;
           the argument after the -t specifies the address to which
           the mail is addressed; and filename is the file that is
           to be sent.  (This file must contain both the header and
           the body of the message.)  The arguments may be given in
           any order.  If the -f and/or -t arguments are missing
           then the addresses are taken from the From: and/or To:
           lines in the header of the message.  If the filename
           is missing then the message is taken from standard
           input.

           The current working directory must contain the file
           WEASEL.INI, because we need to look up that file to
           work out where the "forward" directory is.

   Restriction:
           Requires Weasel version 0.96 or later.

------------------------------------------------------------*/

/****************************************************************/
/*                       MAIN PROGRAM                           */
/****************************************************************/

CALL RxFuncAdd SysLoadFuncs, rexxutil, sysloadfuncs
CALL SysLoadFuncs
Globals = "ReturnPath ForwardPath InFile"

/* Switch our working directory to the program directory */

PARSE SOURCE rubbish rubbish SrcDir
j = LASTPOS('\', SrcDir)
IF j > 0 THEN SrcDir = LEFT(SrcDir, j-1)
WrkDir = DIRECTORY()
CALL DIRECTORY SrcDir

PARSE ARG userargs
CALL ParseArgs userargs

Nul = '00'X
MailRoot = SysIni('WEASEL.INI', '$SYS', 'MailRoot')
j = POS(Nul,MailRoot)
IF j > 0 THEN MailRoot = LEFT(MailRoot, j-1)
MailRoot = STRIP(MailRoot)
MailRoot = TRANSLATE(MailRoot, '\', '/')

TmpFile = SysTempFileName(MailRoot||'forward\?????.tmp')
IF TmpFile = '' THEN
    DO
        SAY "Missing forward directory"
        CALL DIRECTORY WrkDir
        EXIT 1
    END

/* Copy the original input file to a new file, inserting '.'    */
/* fill if needed.  This also lets us deal with input from      */
/* the keyboard.                                                */

IF InFile = '' THEN InFile = 'STDIN'
NewFile = SysTempFileName('?????.tmp')
IF NewFile = '' THEN
    DO
        SAY "Can't create temporary file"
        CALL DIRECTORY WrkDir
        EXIT 1
    END

DO FOREVER
    buffer = LINEIN(InFile)
    IF STREAM(InFile, 'S') \= 'READY' THEN LEAVE
    IF LEFT(buffer, 1) = '.' THEN CALL CHAROUT NewFile, '.'
    CALL LINEOUT NewFile, buffer
END
CALL STREAM NewFile, 'C', 'CLOSE'

InFile = NewFile

ForwardPath = CleanUpAddress(ForwardPath)
ReturnPath = CleanUpAddress(ReturnPath)
IF (ReturnPath = "") | (ForwardPath = "") THEN CALL CheckHeader

    /********************************************************************/
    /* While relay mail is waiting to be sent it is stored as a file    */
    /* which starts with the following details.                         */
    /*   4 bytes   format version, value 'V000'                         */
    /*   4 bytes   send time                                            */
    /*   1 byte    retry number                                         */
    /*   1 byte    Boolean notify-on-failure flag                       */
    /*   variable  sender (character string)                            */
    /*   variable  recipient list, bounded by () and comma-separated    */
    /*                                                                  */
    /* The message content starts immediately after this.               */
    /********************************************************************/

Nul = D2C(0)
CALL CHAROUT TmpFile, 'V000'||Nul||Nul||Nul||Nul||Nul||'1'X
CALL CHAROUT TmpFile, ReturnPath||'('
CALL CHAROUT TmpFile, ForwardPath||')'
CALL stream TmpFile, 'C', 'CLOSE'

'@COPY 'TmpFile'/A+'InFile' 'TmpFile'/B >NUL'
'@RENAME 'TmpFile' 'SysTempFileName('?????.FWD')
'@DEL 'InFile

CALL DIRECTORY WrkDir
EXIT 0

/****************************************************************/
/*              PARSING THE COMMAND-LINE ARGUMENTS              */
/****************************************************************/

ParseArgs: PROCEDURE expose (Globals)
   ReturnPath = ""
   ForwardPath = ""
   InFile = ""
   parse arg head tail
   DO WHILE head \= ""
       IF head = "-f" THEN
              PARSE VAR tail ReturnPath head tail
       ELSE IF head = "-t" THEN
              PARSE VAR tail ForwardPath head tail
       ELSE
           DO
              InFile = head
              PARSE VAR tail head tail
           END
   END
   RETURN

/****************************************************************/
/*      GETTING "To:" AND/OR "From:" LINES FROM THE HEADER      */
/****************************************************************/

CheckHeader: PROCEDURE EXPOSE (Globals)

    InHeader = 1
    FirstLine = 1

    DO WHILE InHeader

        /* Each time around this loop we look at one line of   */
        /* the header.                                         */

        line = LineIn(InFile)
        IF (line = "") & \FirstLine THEN InHeader = 0
        ELSE IF (LEFT(line,1) = ' ') THEN
            DO
               /* Continuation line, do nothing */
            END
        ELSE IF (POS(':', line) = 0) & \FirstLine THEN InHeader = 0
        ELSE
            DO
               PARSE VALUE line WITH keyword ':' remainder
               keyword = TRANSLATE(keyword)
               remainder = CleanUpAddress(remainder)
               IF (keyword = "FROM") & (ReturnPath = "") THEN
                   DO
                       ReturnPath = remainder
                   END
               ELSE IF (keyword = "TO") | (keyword = "CC") | (keyword = "BCC") THEN
                   DO
                       IF ForwardPath = "" THEN ForwardPath = remainder
                       ELSE ForwardPath = ForwardPath","remainder
                   END
            END
        FirstLine = 0
    END
    CALL stream InFile, 'C', 'CLOSE'
    RETURN

/****************************************************************/
/*                 TIDYING UP AN E-MAIL ADDRESS                 */
/****************************************************************/

CleanupAddress: PROCEDURE

    /* The argument to this procedure is an e-mail address from */
    /* the message header.  It has the form user@domain, plus   */
    /* possibly some other detail.  We return the user@domain,  */
    /* stripping off the rest.                                  */

    PARSE ARG line
    line = STRIP(line)
    IF LEFT(line,1) = '"' THEN
        PARSE VAR line '"' rubbish '"' line
    IF RIGHT(line,1) = ')' THEN
        PARSE VAR line line '(' rubbish ')'
    line = STRIP(line)
    IF LEFT(line,1) = '<' THEN
        PARSE VAR line '<' line '>' rubbish
    RETURN line

/****************************************************************/

